import time
from .config import config
from typing import Tuple
from nonebot.adapters import Event

cd = config.whateat_cd
max_count = config.whateat_max


def check_iscd(last_time: int) -> Tuple[bool, int, int]:
    # 检查cd
    current_time = int(time.time())
    delta_time = current_time - last_time
    if delta_time < cd:
        return True, cd - delta_time, last_time
    else:
        return False, 0, current_time


def check_ismax(message: Event, user_count: dict) -> Tuple[bool, dict]:
    # 判断是否达到每日最大值
    user_id = message.get_user_id()
    if max_count == 0:
        return False, {}
    if user_id not in user_count:
        user_count[f"{user_id}"] = 0
    if user_count[f"{user_id}"] < max_count:
        user_count[f"{user_id}"] += 1
        return False, user_count
    else:
        return True, user_count


from nonebot import require

require("nonebot_plugin_alconna")
require("nonebot_plugin_apscheduler")

from nonebot.plugin import PluginMetadata, inherit_supported_adapters
from . import matcher, config, check_on_statup


__plugin_meta__ = PluginMetadata(
    name="今天吃什么（图片版）",
    description="随机发送吃的或者喝的图片",
    usage="""
    今天吃什么
    今天喝什么
    添加菜单
    删除菜单
    查看菜单
    """,
    homepage = "https://github.com/Cvandia/nonebot-plugin-whateat-pic",
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna")
)


from pathlib import Path
from nonebot.log import logger
from .config import config

def save_pic(img: bytes, img_type: str, name:str) -> None:
    """
    保存图片
    """
    save_path = Path(config.whatpic_res_path) / f"{img_type}_pic" / name
    if isinstance(img, bytes):
        with open(save_path, "wb") as f:
            f.write(img)
    else:
        logger.error(f"img must be bytes, but got {type(img)}")
        raise TypeError("img must be bytes")


def delete_pic(img_type: str, name: str) -> None:
    """
    删除图片
    """
    delete_path = Path(config.whatpic_res_path) / f"{img_type}_pic" / name
    if delete_path.exists():
        delete_path.unlink()
    else:
        raise FileNotFoundError(f"{delete_path} not found")



from PIL import Image, ImageDraw, ImageFont
from pathlib import Path
from typing import Literal, Iterator
from .config import config


class Menu:
    '''
    生成菜单的类
    '''
    dish_path: Path # 菜品图片的路径
    menu_background: Image.Image # 菜单背景图
    menu_font: ImageFont.FreeTypeFont # 菜单字体

    def __init__(self, menu_type: Literal["drink", "eat"]) -> None:
        # 菜单图片的路径
        self.dish_path = Path(config.whatpic_res_path) / f"{menu_type}_pic"
        # 获取所有菜品的名字
        self.all_dish_name = [i.stem for i in self.dish_path.iterdir()]
        # 菜单背景图
        self.menu_background = Image.open(Path(__file__).parent / "menu_res" / "menu_bg.jpg")
        # 字体大小
        self.font_size = 30
        # 加载字体
        self.menu_font = ImageFont.truetype(
            str(Path(__file__).parent / "menu_res" / "msyh.ttc"), self.font_size
        )

    @property
    def menu_bg_size(self) -> tuple:
        """
        获取菜单背景图的大小
        """
        return self.menu_background.size

    # 生成迭代器用于for循环遍历所有的生成的菜单
    def draw_menu(self) -> Iterator[Image.Image]:
        # 每个菜单的行数: 背景图高度 - 200(上下边距) // （字体大小+10(行间距)） = 菜单行数
        line_num = (self.menu_bg_size[1] - 150) // (self.font_size + 10)
        # 总共需要合成的图片数量
        img_num = len(self.all_dish_name) // line_num + 1
        # 生成所有菜单的图片
        for i in range(img_num):
            # 生成一张背景图的副本
            menu_img = self.menu_background.copy()
            # 生成draw对象
            draw = ImageDraw.Draw(menu_img)
            # 生成菜单的名字
            for j in range(line_num):
                # 如果菜单名字已经全部生成完毕
                if i * line_num + j >= len(self.all_dish_name):
                    break
                # 生成菜单序号+名字，（居中显示）
                draw.text(
                    ((self.menu_bg_size[0] - 300) // 2, 75 + j * (self.font_size + 10)),
                    f"{i * line_num + j + 1}.{self.all_dish_name[i * line_num + j]}",
                    font=self.menu_font,
                    fill="black",
                )
            yield menu_img
    
    

# 测试程序
# mn = Menu("eat")
# for i in mn.draw_menu():
#     i.show()
#     if input("是否继续") == "n":
#         break



from nonebot.log import logger
from pathlib import Path
from rich.progress import Progress
import asyncio
import json
import httpx
from .config import config

available_urls = [
    "https://raw.githubusercontent.com/Cvandia/nonebot-plugin-whateat-pic/",
    "https://mirror.ghproxy.com/https://raw.githubusercontent.com/Cvandia/nonebot-plugin-whateat-pic/",
    "https://cdn.jsdelivr.net/gh/Cvandia/nonebot-plugin-whateat-pic@",
    "https://fastly.jsdelivr.net/gh/Cvandia/nonebot-plugin-whateat-pic@",
    "https://raw.gitmirror.com/Cvandia/nonebot-plugin-whateat-pic/",
]


async def check_resource():
    semaphore = asyncio.Semaphore(10)
    global available_urls

    async def _download(client: httpx.AsyncClient, name: str):
        async with semaphore:
            for base_url in available_urls:
                url = base_url + "refs/heads/main/res/" + name
                try:
                    resp = await client.get(url, timeout=20, follow_redirects=True)
                    resp.raise_for_status()
                    return resp.content
                except httpx.HTTPError:
                    pass
            logger.warning(f"{url} download failed!")

    async with httpx.AsyncClient() as client:
        if content := await _download(client, "download_list.json"):
            resource_list = json.loads(content.decode("utf-8"))
        else:
            return

    # 下载资源文件
    download_list: list[tuple[Path, str]] = []

    # 添加下载任务
    for item in resource_list["drink_pic"]:
        path_name = "drink_pic/" + item["name"]
        drink_pic_path = Path(config.whatpic_res_path) / path_name
        download_list.append((drink_pic_path, path_name))
    for item in resource_list["eat_pic"]:
        path_name = "eat_pic/" + item["name"]
        eat_pic_path = Path(config.whatpic_res_path) / path_name
        download_list.append((eat_pic_path, path_name))

    if len(download_list):
        logger.info("Downloading images ...")
    else:
        return

    # 下载资源
    async with httpx.AsyncClient() as client:

        async def download_image(file_path: Path, file_name: str):
            if file_path.exists():
                logger.info(f"{file_path} already exists, skipping download.")
                return
            if content := await _download(client, file_name):
                file_path.parent.mkdir(parents=True, exist_ok=True)
                with file_path.open("wb") as f:
                    f.write(content)

        with Progress(
            *Progress.get_default_columns(), "[yellow]{task.completed}/{task.total}"
        ) as progress:
            progress_task = progress.add_task(
                "[green]Downloading...", total=len(download_list)
            )
            tasks = [
                download_image(file_path, file_name)
                for file_path, file_name in download_list
            ]
            for task in asyncio.as_completed(tasks):
                await task
                progress.update(progress_task, advance=1)

from nonebot import get_driver
import asyncio

driver = get_driver()
@driver.on_startup
async def on_startup():
    logger.info("Checking resources...")
    asyncio.create_task(check_resource())

# 测试代码
# if __name__ == "__main__":
#     asyncio.run(check_resource())





from nonebot import get_driver
from nonebot.plugin import get_plugin_config
from pydantic import BaseModel
"""
解决pydantic v2 无法获取配置的问题
"""


class Config(BaseModel):
    
    whateat_cd: int = 10
    whateat_max: int = 0
    whatpic_res_path: str = "./data/whateat_pic"
    
config = get_plugin_config(Config)
